/* checker.c -- Check the memory and return the target board memory layout
   Copyright 2000, 2001 Free Software Foundation, Inc.
   Written by Stephane Carrez (stcarrez@worldnet.fr)

This file is part of GTAM.

GTAM is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

GTAM is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with GTAM; see the file COPYING.  If not, write to
the Free Software Foundation, 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

/*! @page checker Check the memory and return the target board memory layout.

  This 68HC11 bootstrap program checks the target memory to find out
  the memory regions which contain some RAM. It prints out the starting and
  ending addresses of the RAM banks. The output format is:

  <dl>
    <dt>+{ADDR}
    <dd>Indicates that a RAM bank starts at {ADDR} inclusive.
    <dt>-{ADDR}
    <dd>Indicates that a non-RAM bank bank starts at {ADDR} inclusive.
        {ADDR}-1 is the end address of the previous RAM bank.
  </dl>

  Addresses are printed in hexadecimal.

  To find the RAM, we use a basic destructive algorithm:

  <ul>
    <li>read the memory
    <li>invert all the bits and write to the memory
    <li>read again and verify that this matches with what we wrote.
  </ul>

  Once all the memory space is scanned, the program reboots by jumping
  to the internal ROM. The loader is then able to upload another program.

  This bootstrap program is uploaded by the GTAM to obtain the configuration
  of the target board and verify the consistency with the program that must
  be uploaded (if we are uploading some program outside of the 256 bytes
  RAM bank).

  Implementation Notes:

     -# This program must not be linked with a startup file.  It implements
        the startup entry point.
     -# The \c _start function must be at beginning of this file.
        By doing so, it will be mapped at address 0 by the linker and
        we avoid to have some jump to call it (since the boot will
        jump there).
     -# It must be compiled with \b -mshort \b -Os \b -fomit-frame-pointer
        to make sure it fits in less than 248 bytes (keep a few bytes at
        end for the stack).
     -# It must be compiled with \b -msoft-reg-count=0 or =1 to reduce the
        use of soft-registers (since they are mapped in RAM in .page0).
     -# The pointer used to scan the memory is marked \b volatile to prevent
        GCC from optimizing memory accesses.
     -# Before scanning the memory, we switch the bus in the expanded mode.
        We switch back to single chip before rebooting.
     -# The soft registers must be located at beginning of the .page0.
        This will clobber the beginning of the \c _start function.
        We don't care because this clobbers some initialisation part.
     -# Compile with \b -DNO_REBOOT to avoid the reboot.  In that case,
        the program loops indefinately checking the memory regions.

   Caveats:

   <ul>
     <li>It does not detect RAM banks which are mapped at several
         addresses.
   </ul>

  @htmlonly
  Source file: <a href="checker_c-source.html">checker.c</a>
  @endhtmlonly

*/


#include <sys/ports.h>

static void put_addr(unsigned short value);
static void put_char(unsigned char c);

static inline void
reboot()
{
#ifndef NO_REBOOT
  /* The 'func' type is marked as 'noreturn' to optimize the
     generated code.  */
  typedef void __attribute__ ((noreturn)) (* func)();

  func handler;

  set_bus_single_chip ();

  /* To reboot, get the reset vector and go to the start of the ROM.  */
  handler = *((func*) 0xbffe);
  handler ();
#endif
}

void
_start()
{
  unsigned char c;
  volatile unsigned char* addr;
  unsigned char nok = 1;

  set_bus_expanded ();

  /* Start at end of the RAM.  The algorithm is basic and destructive.
     We can't apply it to the RAM where this program is mapped.  */
  addr = (volatile unsigned char*) 0x0ff;
  while (1)
    {
      addr++;

      /* If we reached the end of memory space, flush the SCI by writing
         spurious characters and go back to the ROM.  If we don't print
         some characters, the BREAK generated by the ROM can eat the
         last byte.  */
      if (addr == (unsigned char*) 0)
        {
          for (c = 5; --c != 0; )
            put_char ('\n');

          reboot ();

          /* Skip the .page0 if reboot() is not supported.
             Otherwise, this code is not reached and not generated.  */
          addr += 0xff;
          continue;
        }

      /* Skip the IO registers.  */
      if (addr == _io_ports)
        {
          addr += M6811_IO_SIZE;
          continue;
        }

      /* Check the memory by inverting its content.
         If this succeeds, we have some kind of RAM.  */
      c = ~*addr;
      *addr = c;
      if (c != *addr)
        {
          if (nok)
            continue;

          /* Beginning of non-RAM bank.  */
          nok = 1;
          put_char ('-');
          put_addr ((unsigned short) addr);
        }
      else
        {
          if (nok == 0)
             continue;

          /* Beginning of RAM bank.  */
          nok = 0;
          put_char ('+');
          put_addr ((unsigned short) addr);
        }
    }
}


/* Translate and print the 8-bit value in hexadecimal on the serial line.  */
static void
put_hex(unsigned short value)
{
  put_char ("0123456789ABCDEF"[(value >> 4) & 0x0F]);
  put_char ("0123456789ABCDEF"[value & 0x0F]);
}


/* Translate and print the address on the serial line.  */
static void
put_addr(unsigned short value)
{
  put_hex (value >> 8);
  put_hex (value);
  put_char ('\n');
}


/* Write a character on the serial line.  */
static void
put_char(unsigned char c)
{
  while (!(_io_ports[M6811_SCSR] & M6811_TDRE))
    continue;

  _io_ports[M6811_SCDR] = c;
  _io_ports[M6811_SCCR2] |= M6811_TE;
}
